-- Timing params:
--   Fixed?: False
--   Pipeline Slices: []
--   Input regs?: False
--   Output regs?: False
library std;
use std.textio.all;
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.float_pkg.all;
use work.c_structs_pkg.all;
-- Submodules: 0
entity BIN_OP_PLUS_uint25_t_uint24_t_uint24_t_0CLK_de264c78 is
port(
 left : in unsigned(23 downto 0);
 right : in unsigned(23 downto 0);
 return_output : out unsigned(24 downto 0));
end BIN_OP_PLUS_uint25_t_uint24_t_uint24_t_0CLK_de264c78;
architecture arch of BIN_OP_PLUS_uint25_t_uint24_t_uint24_t_0CLK_de264c78 is
-- Types and such
-- Declarations
constant PIPELINE_LATENCY : integer := 0;

-- One struct to represent this modules variables
type variables_t is record
 -- All of the wires in function

  carry : std_logic_vector(0 downto 0);
  intermediate : std_logic_vector(24 downto 0);
  left_resized : unsigned(23 downto 0);
  right_resized : unsigned(23 downto 0);
  left_range_slv : std_logic_vector(23 downto 0);
  right_range_slv : std_logic_vector(23 downto 0);
  full_width_return_output : unsigned(24 downto 0);
  return_output : unsigned(24 downto 0);
  right : unsigned(23 downto 0);
  left : unsigned(23 downto 0);
end record;

-- Type for this modules register pipeline
type register_pipeline_t is array(0 to PIPELINE_LATENCY) of variables_t;
  -- Each function instance gets signals

begin



 -- Combinatorial process for pipeline stages
process (
 -- Inputs
 left,
 right)
is 
 -- Read and write variables to do register transfers per clock
 -- from the previous to next stage
 variable read_pipe : variables_t;
 variable write_pipe : variables_t;

  -- This modules self pipeline registers read once per clock
  variable read_self_regs : register_pipeline_t;
  variable write_self_regs : register_pipeline_t;
begin
 -- Loop to construct simultaneous register transfers for each of the pipeline stages
 -- LATENCY=0 is combinational Logic
 for STAGE in 0 to PIPELINE_LATENCY loop
  -- Input to first stage are inputs to function
  if STAGE=0 then
   -- Mux in inputs
   read_pipe.left := left;
   read_pipe.right := right;
  else
   -- Default read from previous stage
   read_pipe := read_self_regs(STAGE-1);
  end if;
  -- Default write contents of previous stage
  write_pipe := read_pipe;


  --
  -- One bit adder with carry

  -- width = 24
  -- num_stages = 1
  -- bits per stage = {0: 24}
  
    if STAGE = 0 then
      -- This stuff must be in stage 0
      write_pipe.carry := (others => '0'); -- One bit unsigned
      write_pipe.left_resized := resize(write_pipe.left, 24);
      write_pipe.right_resized := resize(write_pipe.right, 24);
      write_pipe.return_output := (others => '0');
      write_pipe.full_width_return_output := (others => '0');
      
        --  bits_per_stage_dict[0] = 24
        write_pipe.left_range_slv := (others => '0');
        write_pipe.right_range_slv := (others => '0');
        write_pipe.left_range_slv(23 downto 0) := std_logic_vector(write_pipe.left_resized(23 downto 0));
        write_pipe.right_range_slv(23 downto 0) := std_logic_vector(write_pipe.right_resized(23 downto 0));  

        -- Adding unsigned values
        write_pipe.intermediate := (others => '0'); -- Zero out for this stage
        write_pipe.intermediate(24 downto 0) := std_logic_vector( unsigned('0' & write_pipe.left_range_slv(23 downto 0)) + unsigned('0' & write_pipe.right_range_slv(23 downto 0)) );
        -- New carry is msb of intermediate
        write_pipe.carry(0) := write_pipe.intermediate(24);
        -- Assign output bits
        -- Carry full_width_return_output(up_bound+1) will be overidden in next iteration and included as carry
        write_pipe.full_width_return_output(24 downto 0) := unsigned(write_pipe.intermediate(24 downto 0));
      
      write_pipe.return_output := resize(write_pipe.full_width_return_output(24 downto 0), 25);      

    end if;
      -- Write to stage reg
  write_self_regs(STAGE) := write_pipe;
  -- Some tools dont like if read_pipe is never fully driven, dummy drive
  -- And some tools dont like this dummy? read_pipe := write_pipe;
 end loop;


 -- Drive registers and outputs
 -- Last stage of pipeline return wire to return port/reg
 return_output <= write_self_regs(PIPELINE_LATENCY).return_output;
end process;

end arch;
